var __assign = (this && this.__assign) || Object.assign || function(t) {
  for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
          t[p] = s[p];
  }
  return t;
};

var Positioner = /** @class */ (function() {
  function Positioner(innerBounds, outerBounds, height, width) {
    this.innerBounds = innerBounds;
    this.outerBounds = outerBounds;
    this.height = height;
    this.width = width;
    this.orientations = ['top', 'right', 'bottom', 'left'];
    this.hAlignments = ['start', 'center', 'end'];
    this.vAlignments = ['top', 'middle', 'bottom'];
    this.topLane = { name: 'top', x: this.outerBounds.x, y: this.outerBounds.y, width: this.outerBounds.width, height: this.innerBounds.y - this.outerBounds.y };
    this.leftLane = { name: 'left', x: this.outerBounds.x, y: this.outerBounds.y, height: this.outerBounds.height, width: this.innerBounds.x - this.outerBounds.x };
    this.rightLane = { name: 'right', x: this.innerBounds.right, y: this.outerBounds.y, height: this.outerBounds.height, width: this.outerBounds.right - this.innerBounds.right };
    this.bottomLane = { name: 'bottom', x: this.outerBounds.x, y: this.innerBounds.bottom, height: this.outerBounds.bottom - this.innerBounds.bottom, width: this.outerBounds.width };
  }

  Positioner.prototype.top = function() {
    return this.positions('top');
  };
  Positioner.prototype.bottom = function() {
    return this.positions('bottom');
  };
  Positioner.prototype.right = function() {
    return this.positions('right');
  };
  Positioner.prototype.left = function() {
    return this.positions('left');
  };

  Positioner.prototype.lanes = function() {
    return [this.topLane, this.rightLane, this.bottomLane, this.leftLane];
  };

  Positioner.prototype.positions = function(orientation) {
    var _this = this;
    return this.hAlignments.reduce(function(positions, h) {
      return _this.vAlignments.reduce(function(innerPos, v) {
        var alignment = _this.alignedPosition(orientation, h, v);
        return __assign({}, innerPos, ((_a = {}), (_a[h] = __assign({}, innerPos[h], ((_b = {}), (_b[v] = alignment), _b))), (_a[v] = __assign({}, innerPos[v], ((_c = {}), (_c[h] = alignment), _c))), _a));
        var _a, _b, _c;
      }, positions);
    }, {});
  };

  Positioner.prototype.alignedPosition = function(orientation, hAlignment, vAlignment) {
    var bounds = this.boundingBox(orientation, hAlignment);
    var fits = this.fitsBounds(bounds);
    var aligned = this.alignInBounds(bounds, orientation, hAlignment, vAlignment);
    return __assign({}, aligned, { fits: fits });
  };

  Positioner.prototype.boundingBox = function(orientation, alignment) {
    var thisLane = this.relativeLane(orientation, 'center');
    var _a = this.relativeProps(orientation),
      width = _a.width,
      x = _a.x;
    var bounds = __assign({}, thisLane);
    bounds[width] = this.boundingWidth(orientation, alignment);
    bounds[x] = this.boundingX(orientation, alignment, bounds[width]);
    return bounds;
  };

  Positioner.prototype.boundingWidth = function(orientation, alignment) {
    var width = this.relativeProps(orientation).width;
    var endLane = this.relativeLane(orientation, alignment);
    var boundingWidth = this.innerBounds[width] + endLane[width];
    if (alignment === 'center') {
      var extraWidth = boundingWidth - this[width];
      return boundingWidth - extraWidth;
    }
    return boundingWidth;
  };

  Positioner.prototype.boundingX = function(orientation, alignment, boundingWidth) {
    var _a = this.relativeProps(orientation),
      width = _a.width,
      x = _a.x,
      xScale = _a.xScale;
    if ((alignment === 'start' && xScale(1) < 0) || (alignment === 'end' && xScale(1) > 0)) {
      return this.innerBounds[x] + this.innerBounds[width] - boundingWidth;
    } else if (alignment === 'center') {
      return this.innerBounds[x] + (this.innerBounds[width] - this[width]) / 2;
    } else {
      return this.innerBounds[x];
    }
  };

  Positioner.prototype.fitsBounds = function(bounds) {
    return this.height <= bounds.height && this.width <= bounds.width && this.outerBounds.left <= bounds.x && this.outerBounds.top <= bounds.y && this.outerBounds.right >= bounds.x + bounds.width && this.outerBounds.bottom >= bounds.y + bounds.height;
  };

  Positioner.prototype.alignInBounds = function(bounds, orientation, hAlignment, vAlignment) {
    var _a = this.relativeProps(orientation),
      x = _a.x,
      y = _a.y;
    return (_b = { height: this.height, width: this.width }), (_b[x] = this.alignHorizontal(bounds, orientation, hAlignment)), (_b[y] = this.alignVertical(bounds, orientation, vAlignment)), _b;
    var _b;
  };

  Positioner.prototype.alignHorizontal = function(bounds, orientation, alignment) {
    var _a = this.relativeProps(orientation),
      width = _a.width,
      x = _a.x,
      xScale = _a.xScale;
    if ((alignment === 'start' && xScale(1) < 0) || (alignment === 'end' && xScale(1) > 0)) {
      return bounds[x] + bounds[width] - this[width];
    } else if (alignment === 'center') {
      return bounds[x] + (bounds[width] - this[width]) / 2;
    }
    return bounds[x];
  };

  Positioner.prototype.alignVertical = function(bounds, orientation, alignment) {
    var _a = this.relativeProps(orientation),
      height = _a.height,
      y = _a.y,
      yScale = _a.yScale;
    if ((alignment === 'bottom' && yScale(1) < 0) || (alignment === 'top' && yScale(1) > 0)) {
      return bounds[y] + bounds[height] - this[height];
    } else if (alignment === 'middle') {
      return bounds[y] + (bounds[height] - this[height]) / 2;
    }
    return bounds[y];
  };

  Positioner.prototype.relativeProp = function(props, orientation, offset) {
    if (offset === void 0) {
      offset = 0;
    }
    var lanes = this.lanes();
    var orientationIndex = lanes
      .map(function(l) {
        return l.name;
      })
      .indexOf(orientation);
    var propsIndex = (orientationIndex + props.length + offset) % props.length;
    return props[propsIndex];
  };

  Positioner.prototype.relativeProps = function(orientation) {
    var ident = function(x) {
      return x;
    };
    var invert = function(x) {
      return -x;
    };
    return { height: this.relativeProp(['height', 'width'], orientation), width: this.relativeProp(['width', 'height'], orientation), x: this.relativeProp(['x', 'y'], orientation), y: this.relativeProp(['y', 'x'], orientation), xScale: this.relativeProp([ident, ident, invert, invert], orientation), yScale: this.relativeProp([invert, ident, ident, invert], orientation) };
  };

  Positioner.prototype.relativeLane = function(orientation, alignment) {
    var alignmentOffset = { start: 1, center: 0, end: -1 }[alignment];
    return this.relativeProp(this.lanes(), orientation, alignmentOffset);
  };

  return Positioner;
})();

module.exports = Positioner;
